#include "network.h"

static  list<NetWork*> thislist;
pthread_mutex_t thisattr = PTHREAD_MUTEX_INITIALIZER;

static void *read_http(void *arg)
{

	pthread_mutex_lock(&thisattr);

	NetWork *http;
	int	sockfd, len, pos;
	string str;
	string data;

	http = ((ARG*)arg)->network;
	sockfd = ((ARG*)arg)->sockfd;

	if(sockfd < -1) {
		close(sockfd);
		delete ((ARG*)arg);
		cout << "not connect?" << endl;
		pthread_mutex_unlock(&thisattr);
		pthread_exit((void*)false);
	}
	char buf[512] = {'\0'};
	data.clear();
	while( (len = recv(sockfd, buf, sizeof(buf) - sizeof(char), 0)) > 0) {
		buf[len] = '\0';
		data.append(buf, len);
	}

	pos = http->data_to_ResponseHead(data);
	http->data_to_ResponseText(data, pos);

	thislist.push_back(http);
	pthread_mutex_unlock(&thisattr);
	close(sockfd);

	pthread_kill(((ARG*)arg)->pid, EMITSIGNAL);
	delete ((ARG*)arg);
	pthread_exit((void*)true);
}

static void emit(int signo)
{
	pthread_mutex_lock(&thisattr);
	if(thislist.empty()) {
		pthread_mutex_unlock(&thisattr);
		return;
	}
	if(signo != EMITSIGNAL) {
		pthread_mutex_unlock(&thisattr);
		return ;
	}

	NetWork*	temp;
	do {
		temp = thislist.front();

		temp->emit_signal(temp->get_Response_text());
		//temp->get_Response_text()->clear();
		thislist.remove(temp);
	}while(!thislist.empty());
		
	pthread_mutex_unlock(&thisattr);
	
}
NetWork::NetWork(string url)
{
	if(!url.empty()) {
		set_url(url);
	}
	signal(EMITSIGNAL, emit);
}

NetWork::~NetWork()
{
}

bool NetWork::get()
{
	int sockfd;
	if( (sockfd = Connect_to_server()) <= 0) {
		return false;
	}

	int i = 0;
	for( ; i < 3; ++i) {
		if(send_server(sockfd)) {
			break;
		}
	}
	if(i == 3) {
		cout << "The last attempt a failure" << endl;
		return false;
	}

	recv_server(sockfd);
	return true;
}

int NetWork::Connect_to_server()
{
	int	err_code, sockfd;
	struct addrinfo hints, *res, *resave;

	if(m_host.empty()) {
		cout << "m_host is empty\n";
		return false;
	}
	bzero(&hints, sizeof(struct addrinfo));
	hints.ai_family = AF_INET;
	hints.ai_socktype = SOCK_STREAM;

	if( (err_code = getaddrinfo(m_host.c_str(), "80", &hints, &res)) != 0) {
		cout << "getaddrinfo error: " << gai_strerror(err_code) << endl;
		return false;
	}
	resave = res;

	do {
		if( (sockfd = socket(res->ai_family, res->ai_socktype, res->ai_protocol)) < 0) {
			continue;
		}

		if(connect(sockfd, res->ai_addr, res->ai_addrlen) == 0) {
			break;
		}

		close(sockfd);
	} while( (res = res->ai_next) != NULL);

	freeaddrinfo(resave);
	if(res == NULL) {
		cout << "connect error for " << m_host << endl;
		return false;
	}

	return sockfd;
}

bool NetWork::send_server(int sockfd)
{
	if(m_requestHead.empty()) {
		cout << "Request Head is emptr" << endl;
		return false;
	}
	m_requestHead += "\r\n";

	int size = m_requestHead.size();
	if(send(sockfd, m_requestHead.c_str(), size, 0) < size) {
		cout << "Write the data does not meet the requirements" << endl;
		return false;
	}
	return true;
}
void NetWork::create_requestHead()
{
	m_requestHead =  Glib::ustring::compose("GET %1 HTTP/1.1\r\n", m_path);
	m_requestHead +=  Glib::ustring::compose("Host: %1\r\n", m_host);
	m_requestHead +=  "User-Agent: Baidu music Downloads (Linux)\r\n";
}

bool NetWork::set_url(const string &url)
{
	if(url.empty()) {
		return false;
	}
	if(!analysis_url(url)) {
		return false;
	}
	create_requestHead();
	return true;
}

void NetWork::emit_signal(const string &data)
{
	m_signal_something.emit(data);
}

void NetWork::append_request_head(const string &headerName, const string &headerValue)
{
	if(m_requestHead.empty()) {
		cout << "m_requesrHead.empty() return true" << endl;
		return;
	}
	m_requestHead += Glib::ustring::compose("%1: %2\r\n", headerName, headerValue);
}

NetWork::type_signal_something NetWork::signal_something()
{
	return m_signal_something;
}

bool NetWork::analysis_url(const string &url)
{
	int pos = 0;

	if( (pos = url.find("http://", 0)) == string::npos) {
		return false;
	}
	pos += Strcpy(&m_host, url, pos += 7, '/');

	Strcpy(&m_path, url, pos, '\0');

	return true;
}

int Strcpy(string *dst, const string &src, int pos, char end)
{
	int count = 0, size = 0;

	dst->clear();
	size = src.size();
	while(src[pos] != end && count < size) {
		*dst += src[pos++];
		count++;
	}

	return count;
}

string NetWork::get_Response_head()
{
	return m_ResponseHead;
}

string NetWork::get_Response_text()
{
	return m_ResponseText;
}

int NetWork::get_Response_sign(const string &sign, string *ret)
{
	if(m_ResponseHead.empty()) {
		cout << "Response is empty" << endl;
		return -1;
	}
	int pos = 0;

	if( (pos = m_ResponseHead.find(sign, 0)) == string::npos) {
		return true;
	}
	Strcpy(ret, m_ResponseHead, pos += sign.size() + 2, '\r');

	return pos;
}

int NetWork::get_Response_code()
{
	if(m_ResponseHead.empty()) {
		cout << "Response is empty" << endl;
		return -1;
	}

	string str;
	int pos = 0, code;

	if( (pos = m_ResponseHead.find(" ", 0)) == string::npos) {
		cout << "Not found response code" << endl;
		return -2;
	}
	Strcpy(&str, m_ResponseHead, pos, 20);
	sscanf(str.c_str(), "%d", &code);

	return code;
}

void NetWork::recv_server(int sockfd)
{
	ARG *arg = new ARG;
	arg->network = this;
	arg->pid = pthread_self();
	arg->sockfd = sockfd;

	pthread_t pid;
	pthread_create(&pid, NULL, read_http, (void*)arg);
}

int NetWork::data_to_ResponseHead(const string &data)
{
	int pos = 0;

	if( (pos = data.find("\r\n\r\n", 0)) == string::npos) {
		return -1;
	}
	m_ResponseHead.clear();
	m_ResponseHead.assign(data, 0, pos);
	pos += 4;

	return pos;
}

int NetWork::data_to_ResponseText(const string &data, int pos)
{
	string num, str;
	int ret, n;

	m_ResponseText.clear();
	ret = get_Response_sign("Transfer-Encoding", &str);
	if(ret > 0 && str == "chunked") {
		do {
			if(Strcpy(&num, data, pos, '\r') <= 0) {
				break;
			}
			sscanf(num.c_str(), "%x", &n);
			if(n <= 0) {
				break;
			}
			m_ResponseText.append(data, pos += num.size() + 2, n);
			//sbuf.assign(data, pos += num.size() + 2, n);
			//cout << sbuf ;
			//(*http->get_Response_text()) += sbuf;
			pos += n;
		} while(/*(data.find("\r\n", pos) != string::npos) &&*/(pos += 2));
	} else {
		m_ResponseText.assign(data, pos, data.size());
	}

	return pos;
}
